@@ -1,8 +1,9 @@
 """Helpers for usage with `Tornado <https://www.tornadoweb.org>`_"""
+from h5grove.utils import PathError
 import os
-from typing import Optional
+from typing import Any, Optional
 import h5py
-from tornado.web import RequestHandler, MissingArgumentError
+from tornado.web import RequestHandler, MissingArgumentError, HTTPError
 from .content import DatasetContent, ResolvedEntityContent, create_content
 from .encoders import encode
 
@@ -24,13 +25,16 @@ class BaseHandler(RequestHandler):
     def get(self):
         file_path = self.get_query_argument("file")
         if file_path is None:
-            raise MissingArgumentError("File argument is required")
+            raise MissingArgumentError("file")
 
         path = self.get_query_argument("path", None)
         format = self.get_query_argument("format", None)
 
         with h5py.File(os.path.join(self.base_dir, file_path), "r") as h5file:
-            content = self.get_content(h5file, path)
+            try:
+                content = self.get_content(h5file, path)
+            except PathError as e:
+                raise HTTPError(status_code=404, reason=str(e))
 
         encoded_content_chunks, headers = encode(content, format)
 
@@ -47,6 +51,10 @@ class BaseHandler(RequestHandler):
         if self.allow_origin is not None:
             self.set_header("Access-Control-Allow-Origin", self.allow_origin)
 
+    def write_error(self, status_code: int, **kwargs: Any) -> None:
+        self.prepare()
+        return super().write_error(status_code, **kwargs)
+
 
 class AttributeHandler(BaseHandler):
     def get_content(self, h5file, path):
@@ -66,7 +74,13 @@ class DataHandler(BaseHandler):
 
 class MetadataHandler(BaseHandler):
     def get_content(self, h5file, path):
-        content = create_content(h5file, path)
+        resolve_links_arg = self.get_query_argument("resolve_links", None)
+        resolve_links = (
+            resolve_links_arg.lower() != "false"
+            if resolve_links_arg is not None
+            else True
+        )
+        content = create_content(h5file, path, resolve_links)
         return content.metadata()
 
 
